import { onMounted, onUnmounted, ref, isRef, computed } from '@vue/composition-api';

function getQuery(min, max) {
    if (min === void 0) { min = 0; }
    if (max === void 0) { max = 0; }
    if (!min && !max) {
        throw new Error("Please specify at least one value, either 'min' or 'max'");
    }
    else if (min && !max) {
        return "only screen and (min-width: " + min + "px)";
    }
    else if (!min && max) {
        return "only screen and (max-width: " + (max - 1) + "px)";
    }
    else {
        return "only screen and (min-width: " + min + "px) and (max-width: " + (max - 1) + "px)";
    }
}

var defaultEvents = ['mousedown', 'touchstart'];
function useClickAway(elRef, onClickAway, events) {
    if (events === void 0) { events = defaultEvents; }
    var handler = function (e) {
        if (elRef.value && !elRef.value.contains(e.target)) {
            onClickAway(e);
        }
    };
    onMounted(function () {
        events.forEach(function (evtName) { return document.addEventListener(evtName, handler); });
    });
    onUnmounted(function () {
        events.forEach(function (evtName) { return document.removeEventListener(evtName, handler); });
    });
}

function useMedia(query, defaultState) {
    if (defaultState === void 0) { defaultState = false; }
    var mql;
    var matches = ref(defaultState);
    var updateMatchValue = function () { return (matches.value = mql.matches); };
    onMounted(function () {
        mql = window.matchMedia(query);
        mql.addListener(updateMatchValue);
        updateMatchValue();
    });
    onUnmounted(function () {
        mql.removeListener(updateMatchValue);
    });
    return matches;
}

function useMouse() {
    var docX = ref(0);
    var docY = ref(0);
    var mouseMoveHandler = function (e) {
        docX.value = e.pageX;
        docY.value = e.pageY;
    };
    onMounted(function () {
        document.addEventListener('mousemove', mouseMoveHandler);
    });
    onUnmounted(function () {
        document.removeEventListener('mousemove', mouseMoveHandler);
    });
    return {
        docX: docX,
        docY: docY
    };
}

function useMouseElement(elRef) {
    var docX = ref(0);
    var docY = ref(0);
    var elX = ref(0);
    var elY = ref(0);
    var elInfoX = ref(0);
    var elInfoY = ref(0);
    var elInfoW = ref(0);
    var elInfoH = ref(0);
    var mouseMoveHandler = function (e) {
        if (!elRef.value)
            return;
        docX.value = e.pageX;
        docY.value = e.pageY;
        var _a = elRef.value.getBoundingClientRect(), left = _a.left, top = _a.top, height = _a.height, width = _a.width;
        elInfoX.value = left + window.pageXOffset;
        elInfoY.value = top + window.pageYOffset;
        elInfoW.value = width;
        elInfoH.value = height;
        elX.value = e.pageX - elInfoX.value;
        elY.value = e.pageY - elInfoY.value;
    };
    onMounted(function () {
        document.addEventListener('mousemove', mouseMoveHandler);
    });
    onUnmounted(function () {
        document.removeEventListener('mousemove', mouseMoveHandler);
    });
    return {
        docX: docX,
        docY: docY,
        elX: elX,
        elY: elY,
        elInfoX: elInfoX,
        elInfoY: elInfoY,
        elInfoW: elInfoW,
        elInfoH: elInfoH
    };
}

function useHover(elRef) {
    var isHovered = ref(false);
    var mouseEnterHandler = function () {
        isHovered.value = true;
    };
    var mouseLeaveHandler = function () {
        isHovered.value = false;
    };
    onMounted(function () {
        if (!elRef.value)
            return;
        elRef.value.addEventListener('mouseenter', mouseEnterHandler);
        elRef.value.addEventListener('mouseleave', mouseLeaveHandler);
    });
    onUnmounted(function () {
        if (!elRef.value)
            return;
        elRef.value.removeEventListener('mouseenter', mouseEnterHandler);
        elRef.value.removeEventListener('mouseleave', mouseLeaveHandler);
    });
    return isHovered;
}

var getFps = function (fps) { return Number(isRef(fps) ? fps.value : fps); };
var calcFpsInterval = function (fps) { return 1000 / fps; };
var fpsLimit = 60;
function useRafFn(callback, 
// Note: a value equal to or greater than 60 will disable the fps check logic
// giving maximum precision and smoothness
fps, runOnMount) {
    if (fps === void 0) { fps = fpsLimit; }
    if (runOnMount === void 0) { runOnMount = true; }
    var isRunning = ref(false);
    // Using a computed here allows us to update fps
    // dynamically from user's input
    var fpsInterval = computed(function () { return calcFpsInterval(getFps(fps)); });
    var itWasIdle = false;
    var startTime = 0;
    var timeElapsed = 0;
    var timePausedTot = 0;
    var timeDelta = 0;
    var loop = function (timeStamp) {
        if (!startTime)
            startTime = timeStamp;
        if (!isRunning.value)
            return;
        // Store the time that the loop is idle so that we can
        // calculate the timeElapsed variable correctly and return
        // it to the user from within the callback
        if (itWasIdle) {
            timePausedTot = timeStamp - startTime - timeElapsed;
            itWasIdle = false;
        }
        // Adjust timeElapsed to account for startTime and timePausedTot
        timeElapsed = timeStamp - startTime - timePausedTot;
        // Always run the callback if fps is greater than fpsLimit
        var callbackShouldAlwaysRun = getFps(fps) >= fpsLimit;
        if (callbackShouldAlwaysRun) {
            // Store timeDelta for future computations
            timeDelta = timeElapsed;
            callback(timeElapsed);
        }
        // Run callback only when !callbackShouldAlwaysRun
        // and the given fps matches the lapsed time
        if (!callbackShouldAlwaysRun) {
            var elapsedTimeFromPrevLoop = Math.ceil(timeElapsed - timeDelta);
            if (elapsedTimeFromPrevLoop > fpsInterval.value) {
                // Store timeDelta for future computations
                timeDelta = timeElapsed;
                callback(timeElapsed);
            }
        }
        // Run loop again recursively
        requestAnimationFrame(loop);
    };
    var start = function () {
        isRunning.value = true;
        requestAnimationFrame(loop);
    };
    var stop = function () {
        isRunning.value = false;
        itWasIdle = true;
    };
    onMounted(function () { return runOnMount && start(); });
    onUnmounted(stop);
    return {
        isRunning: isRunning,
        start: start,
        stop: stop
    };
}

var fpsLimit$1 = 60;
function useRaf(fps, runOnMount) {
    if (fps === void 0) { fps = fpsLimit$1; }
    if (runOnMount === void 0) { runOnMount = true; }
    var elapsed = ref(0);
    var animHandler = function (elapsedTime) {
        elapsed.value = elapsedTime;
    };
    var _a = useRafFn(animHandler, fps, runOnMount), isRunning = _a.isRunning, start = _a.start, stop = _a.stop;
    return {
        isRunning: isRunning,
        elapsed: elapsed,
        start: start,
        stop: stop
    };
}

function useTimeoutFn(callback, ms, runOnMount) {
    if (ms === void 0) { ms = 0; }
    if (runOnMount === void 0) { runOnMount = true; }
    var isReady = ref(false);
    var isIdle = ref(!runOnMount);
    var timeout = null;
    var cancel = function () {
        isReady.value = null;
        if (timeout) {
            clearTimeout(timeout);
            timeout = null;
        }
    };
    var setTimer = function () {
        isReady.value = false;
        if (isIdle.value)
            isIdle.value = false;
        if (timeout)
            clearTimeout(timeout);
        timeout = setTimeout(function () {
            isReady.value = true;
            callback();
            timeout = null;
        }, ms);
    };
    onMounted(function () { return runOnMount && setTimer(); });
    onUnmounted(cancel);
    return { isReady: isReady, isIdle: isIdle, cancel: cancel, start: setTimer };
}

var noop = function () { return null; };
function useTimeout(ms, runOnMount) {
    if (ms === void 0) { ms = 0; }
    if (runOnMount === void 0) { runOnMount = true; }
    var _a = useTimeoutFn(noop, ms, runOnMount), isReady = _a.isReady, isIdle = _a.isIdle, cancel = _a.cancel, start = _a.start;
    return { isReady: isReady, isIdle: isIdle, cancel: cancel, start: start };
}

export { getQuery, useClickAway, useMedia, useMouse, useMouseElement, useHover, useRafFn, useRaf, useTimeoutFn, useTimeout };
